@@ -2,6 +2,7 @@ from dataclasses import dataclass, field
 from enum import Enum
 from dataclasses_json import config, dataclass_json
 import typing as t
+from .utils import read_pvsyst_file
 
 from tyba_client.utils import string_enum
 
@@ -75,6 +76,99 @@ class PVModule(object):
     bifaciality: float
     bifacial_ground_clearance_height: float
 
+
+@string_enum
+class MermoudModuleTech(Enum):
+    SiMono = "mtSiMono"
+    SiPoly = "mtSiPoly"
+    CdTe = "mtCdTe"
+    CIS = "mtCIS"
+    uCSi_aSiH = "mtuCSi_aSiH"
+
+
+@dataclass_json
+@dataclass
+class PVModuleMermoudLejeune(object):
+    bifacial: bool
+    bifacial_transmission_factor: float
+    bifaciality: float
+    bifacial_ground_clearance_height: float
+    tech: MermoudModuleTech
+    i_mp_ref: float
+    i_sc_ref: float
+    length: float
+    n_diodes: int
+    n_parallel: int
+    n_series: int
+    r_s: float
+    r_sh_0: float
+    r_sh_exp: float
+    r_sh_ref: float
+    s_ref: float
+    t_c_fa_alpha: float
+    t_ref: float
+    v_mp_ref: float
+    v_oc_ref: float
+    width: float
+    alpha_sc: float
+    beta_oc: float
+    mu_n: float
+    n_0: float
+    iam_c_cs_iam_value: t.Optional[t.List[float]] = opt_field()
+    iam_c_cs_inc_angle: t.Optional[t.List[float]] = opt_field()
+    custom_d2_mu_tau: t.Optional[float] = opt_field()
+
+    @classmethod
+    def from_pan(cls, pan_file: str):
+        pan_blob = read_pvsyst_file(pan_file)
+        data = pan_blob["PVObject_"]["items"]
+        commercial = data["PVObject_Commercial"]["items"]
+        if "PVObject_IAM" in data:
+            iam_points = [
+                v.split(",")
+                for k, v in data["PVObject_IAM"]["items"]["IAMProfile"]["items"].items()
+                if k.startswith("Point_")
+            ]
+            iam_angles = [float(v[0]) for v in iam_points]
+            iam_values = [float(v[1]) for v in iam_points]
+        else:
+            iam_angles = None
+            iam_values = None
+
+        return cls(
+            bifacial="BifacialityFactor" in data,
+            bifacial_transmission_factor=0.013,
+            bifaciality=float(data.get("BifacialityFactor", 0.65)),
+            bifacial_ground_clearance_height=1.0,
+            n_parallel=int(data["NCelP"]),
+            n_diodes=int(data["NDiode"]),
+            n_series=int(data["NCelS"]),
+            t_ref=float(data["TRef"]),
+            s_ref=float(data["GRef"]),
+            i_sc_ref=float(data["Isc"]),
+            v_oc_ref=float(data["Voc"]),
+            i_mp_ref=float(data["Imp"]),
+            v_mp_ref=float(data["Vmp"]),
+            alpha_sc=float(data["muISC"]) * 1e-3,  # TODO: check units
+            beta_oc=float(data["muVocSpec"]) * 1e-3,
+            n_0=float(data["Gamma"]),
+            mu_n=float(data["muGamma"]),
+            r_sh_ref=float(data["RShunt"]),
+            r_s=float(data["RSerie"]),
+            r_sh_0=float(data["Rp_0"]),
+            r_sh_exp=float(data["Rp_Exp"]),
+            tech=data["Technol"],
+            length=float(commercial["Height"]),
+            width=float(commercial["Width"]),
+            # faiman cell temp model used by PVSyst
+            t_c_fa_alpha=float(data["Absorb"]),
+            # IAM
+            iam_c_cs_iam_value=iam_values,
+            iam_c_cs_inc_angle=iam_angles,
+            custom_d2_mu_tau=data.get("D2MuTau"),
+        )
+
+
 @dataclass_json
 @dataclass
 class Losses(object):
@@ -138,13 +232,15 @@ class SolarResource(object):
 class PVModel(object):
     solar_resource: t.Union[t.Tuple[float, float], SolarResource]
     inverter: t.Union[str, Inverter]
-    pv_module: t.Union[str, PVModule]
+    pv_module: t.Union[str, t.Union[PVModule, PVModuleMermoudLejeune]]
     system_design: SystemDesign
     losses: t.Optional[Losses] = opt_field()
     layout: t.Optional[Layout] = opt_field()
     project_term: t.Optional[int] = None
     dc_degradation: t.Optional[float] = None
 
+DetailedPVModel = PVModel
+
 @dataclass_json
 @dataclass
 class SingleStorageInputs(object):
@@ -170,6 +266,7 @@ class Battery(object):
 class StorageInputs(object):
     batteries: t.List[Battery]
     cycling_cost_adder: t.Optional[float] = 0
+    annual_cycle_limit: t.Optional[float] = opt_field()
     window: t.Optional[int] = opt_field()
     step: t.Optional[int] = opt_field()
 
@@ -214,4 +311,16 @@ class PVStorageModel(object):
     storage_coupling: StorageCoupling = field(metadata=StorageCoupling.__metadata__)
     pv_inputs: PVModel
     storage_inputs: StorageInputs
-    energy_prices: t.List[float]
\ No newline at end of file
+    energy_prices: t.List[float]
+
+
+@string_enum
+class Market:
+    RT = 'realtime'
+    DA = 'dayahead'
+
+@string_enum
+class AncillaryService:
+    REGULATION_UP = 'Regulation Up'
+    REGULATION_DOWN = 'Regulation Down'
+    RESERVES = 'Reserves'
